import pytest

from lockhart.prompts import load_prompt


@pytest.fixture
def config():
    return {
        "prompts": {
            "prompt1": {"profile": "profile1", "key1": "value1"},
            "prompt2": {"profile": "profile2", "key2": "value2"},
            "prompt3": {"key1": "value1"},
        },
        "profiles": {"profile1": {"key3": "value3"}, "profile2": {"key4": "value4"}},
    }


def test_load_prompt(config, mocker):
    mock_config = mocker.patch("lockhart.prompts.config")
    mock_config.__getitem__.side_effect = config.__getitem__
    prompt1_config = load_prompt("prompt1")
    assert prompt1_config == {"key1": "value1", "key3": "value3"}

    prompt2_config = load_prompt("prompt2")
    assert prompt2_config == {"key2": "value2", "key4": "value4"}

    prompt3_config = load_prompt("prompt3")
    assert prompt3_config == {"key1": "value1"}



# SPDX-FileCopyrightText: 2023-present Waylon S. Walker <waylon@waylonwalker.com>
#
# SPDX-License-Identifier: MIT


import typer

from lockhart.cli.common import verbose_callback
from lockhart.cli.config import config_app
from lockhart.cli.history import history_app
from lockhart.cli.prompt import prompt_app
from lockhart.cli.tui import tui_app

app = typer.Typer(
    name="lockhart",
    help="Lockhart codegen",
)


def version_callback(value: bool) -> None:
    """Callback function to print the version of the lockhart package.

    Args:
        value (bool): Boolean value to determine if the version should be printed.

    Raises:
        typer.Exit: If the value is True, the version will be printed and the program will exit.

    Example:
        version_callback(True)
    """
    if value:
        from lockhart.__about__ import __version__

        typer.echo(f"{__version__}")
        raise typer.Exit()


@app.callback()
def main(
    version: bool = typer.Option(
        None,
        "--version",
        callback=version_callback,
        is_eager=True,
    ),
    verbose: bool = typer.Option(
        False,
        callback=verbose_callback,
        help="show the log messages",
    ),
) -> None:
    return


app.add_typer(config_app)
app.add_typer(prompt_app)
app.add_typer(history_app)
app.add_typer(tui_app)



import datetime
from unittest.mock import patch

import pytest

HISTORY = []

HISTORY.append(
    {
        "prompt_name": "create-history",
        "datetime": datetime.datetime(2023, 1, 22, 10, 24, 8, 977243),
        "request": {
            "engine": "code-davinci-edit-001",
            "temperature": 0,
            "top_p": 1.0,
            "instruction": "write a python script to pickle a python list into a file called ~/.config/lockhart/history.pkl",
            "input": "",
        },
        "response": {
            "choices": [
                {
                    "index": 0,
                    "text": '#!/usr/bin/env python\n\nimport pickle\nimport os\n\ndef save_history(history):\n    """\n    Save the history to a file.\n    """\n    home = os.path.expanduser("~")\n    config_dir = os.path.join(home, ".config", "lockhart")\n    if not os.path.exists(config_dir):\n        os.makedirs(config_dir)\n    history_file = os.path.join(config_dir, "history.pkl")\n    with open(history_file, "wb") as f:\n        pickle.dump(history, f)\n\ndef load_history():\n    """\n    Load the history from a file.\n    """\n    home = os.path.expanduser("~")\n    config_dir = os.path.join(home, ".config", "lockhart")\n    history_file = os.path.join(config_dir, "history.pkl")\n    if os.path.exists(history_file):\n        with open(history_file, "rb") as f:\n            return pickle.load(f)\n    else:\n        return []\n\nif __name__ == "__main__":\n    history = load_history()\n    history.append("test")\n    save_history(history)\n',
                }
            ],
            "created": 1674404634,
            "object": "edit",
            "usage": {
                "completion_tokens": 356,
                "prompt_tokens": 35,
                "total_tokens": 391,
            },
        },
    }
)


HISTORY.append(
    {
        "prompt_name": "create-function",
        "datetime": datetime.datetime(2023, 2, 15, 12, 15, 8, 977243),
        "request": {
            "engine": "code-davinci-edit-001",
            "temperature": 0,
            "top_p": 1.0,
            "instruction": "write a python function to calculate the area of a circle",
            "input": "",
        },
        "response": {
            "choices": [
                {
                    "index": 0,
                    "text": '#!/usr/bin/env python\n\ndef area_of_circle(radius):\n    """\n    Calculate the area of a circle given the radius.\n    """\n    return 3.14 * (radius ** 2)\n',
                }
            ],
            "created": 1674404634,
            "object": "edit",
            "usage": {
                "completion_tokens": 356,
                "prompt_tokens": 35,
                "total_tokens": 391,
            },
        },
    }
)

HISTORY.append(
    {
        "prompt_name": "create-class",
        "datetime": datetime.datetime(2023, 3, 10, 14, 30, 8, 977243),
        "request": {
            "engine": "code-davinci-edit-001",
            "temperature": 0,
            "top_p": 1.0,
            "instruction": "write a python class to represent a person",
            "input": "",
        },
        "response": {
            "choices": [
                {
                    "index": 0,
                    "text": '#!/usr/bin/env python\n\nclass Person:\n    """\n    A class to represent a person.\n    """\n    def __init__(self, name, age):\n        self.name = name\n        self.age = age\n\n    def say_hello(self):\n        print("Hello, my name is {} and I am {} years old.".format(self.name, self.age))\n',
                }
            ],
            "created": 1674404634,
            "object": "edit",
            "usage": {
                "completion_tokens": 356,
                "prompt_tokens": 35,
                "total_tokens": 391,
            },
        },
    }
)

HISTORY.append(
    {
        "prompt_name": "create-dictionary",
        "datetime": datetime.datetime(2023, 4, 5, 16, 45, 8, 977243),
        "request": {
            "engine": "code-davinci-edit-001",
            "temperature": 0,
            "top_p": 1.0,
            "instruction": "write a python script to create a dictionary of key-value pairs",
            "input": "",
        },
        "response": {
            "choices": [
                {
                    "index": 0,
                    "text": '#!/usr/bin/env python\n\ndef create_dictionary(keys, values):\n    """\n    Create a dictionary of key-value pairs.\n    """\n    return dict(zip(keys, values))\n',
                }
            ],
            "created": 1674404634,
            "object": "edit",
            "usage": {
                "completion_tokens": 356,
                "prompt_tokens": 35,
                "total_tokens": 391,
            },
        },
    }
)

HISTORY.append(
    {
        "prompt_name": "create-list",
        "datetime": datetime.datetime(2023, 5, 1, 18, 0, 8, 977243),
        "request": {
            "engine": "code-davinci-edit-001",
            "temperature": 0,
            "top_p": 1.0,
            "instruction": "write a python script to create a list of numbers from 1 to 10",
            "input": "",
        },
        "response": {
            "choices": [
                {
                    "index": 0,
                    "text": '#!/usr/bin/env python\n\ndef create_list():\n    """\n    Create a list of numbers from 1 to 10.\n    """\n    return list(range(1, 11))\n',
                }
            ],
            "created": 1674404634,
            "object": "edit",
            "usage": {
                "completion_tokens": 356,
                "prompt_tokens": 35,
                "total_tokens": 391,
            },
        },
    }
)


HISTORY.append(
    {
        "prompt_name": "create-tuple",
        "datetime": datetime.datetime(2023, 6, 20, 20, 15, 8, 977243),
        "request": {
            "engine": "code-davinci-edit-001",
            "temperature": 0,
            "top_p": 1.0,
            "instruction": "write a python script to create a tuple of numbers from 1 to 10",
            "input": "",
        },
        "response": {
            "choices": [
                {
                    "index": 0,
                    "text": '#!/usr/bin/env python\n\ndef create_tuple():\n    """\n    Create a tuple of numbers from 1 to 10.\n    """\n    return tuple(range(1, 11))\n',
                }
            ],
            "created": 1674404634,
            "object": "edit",
            "usage": {
                "completion_tokens": 356,
                "prompt_tokens": 35,
                "total_tokens": 391,
            },
        },
    }
)

HISTORY.append(
    {
        "prompt_name": "create-string",
        "datetime": datetime.datetime(2023, 7, 10, 22, 30, 8, 977243),
        "request": {
            "engine": "code-davinci-edit-001",
            "temperature": 0,
            "top_p": 1.0,
            "instruction": "write a python script to create a string of characters",
            "input": "",
        },
        "response": {
            "choices": [
                {
                    "index": 0,
                    "text": '#!/usr/bin/env python\n\ndef create_string(characters):\n    """\n    Create a string of characters.\n    """\n    return \'\'.join(characters)\n',
                }
            ],
            "created": 1674404634,
            "object": "edit",
            "usage": {
                "completion_tokens": 356,
                "prompt_tokens": 35,
                "total_tokens": 391,
            },
        },
    }
)


@pytest.fixture()
def load_history():
    with patch("lockhart.tui.app.load_history", return_value=HISTORY) as f:
        yield f



#!/usr/bin/env python

import pickle
from pathlib import Path

from lockhart.console import console


def save_history(history, history_file=None):
    """Save the history to a file."""
    if history_file is None:
        history_file = get_history_file()
    if isinstance(history_file, str):
        history_file = Path(history_file)
    console.log(f"saving\n{history} to {history_file}")
    with history_file.open("wb") as f:
        pickle.dump(history, f)
    console.log(f"wrote history")


def load_history(history_file=None):
    """Load the history from a file."""
    if history_file is None:
        history_file = get_history_file()
    if isinstance(history_file, str):
        history_file = Path(history_file)
    if history_file.exists():
        with history_file.open("rb") as f:
            history = pickle.load(f)
            console.log(f"loaded {len(history)} items into history")
            return history
    else:
        console.log(f"history_file does not exist, returning empty history")
        return []


def get_history_file():
    config_dir = Path.home() / ".config" / "lockhart"
    config_dir.mkdir(parents=True, exist_ok=True)
    history_file = config_dir / "history.pkl"
    return history_file


if __name__ == "__main__":
    history = load_history()
    history.append("test")
    save_history(history)



from pathlib import Path

import pyperclip
from textual.app import App, ComposeResult
from textual.containers import Container
from textual.css.query import NoMatches
from textual.reactive import reactive
from textual.widgets import Button, Footer, Static

from lockhart.config import config
from lockhart.console import console
from lockhart.history import load_history
from lockhart.prompts import run_prompt

REQUEST_KEYS = ["engine", "temperature", "instruction", "input", "prompt"]
RESPONSE_KEYS = ["created", "choices", "usage"]


class PromptSidebar(Static):
    def compose(self) -> ComposeResult:
        key = [
            b
            for b in config["lockhart"]["tui"]["bindings"]
            if b["action"] == "toggle_sidebar"
        ][-1]["key"]
        yield Container(
            Static(f"stored prompts ({key})"),
            *[
                Button(prompt.replace("-", " ").title(), id=f"{prompt}-button")
                for prompt in config["prompts"].keys()
            ],
            id="prompt-sidebar",
        )


class Request(Static):
    """A Request widget."""

    data = reactive({})
    myid = reactive(0)

    def __init__(self, id, data, classes=None):
        super().__init__(classes=classes)
        self.data = data
        self.myid = id

    def update_data(self):
        self.log("updating data")
        if self.data is None:
            return
        self.query_one("#id", Static).update(str(self.myid))
        self.query_one("#date", Static).update(str(self.data["datetime"]))
        for key in REQUEST_KEYS:
            value = str(self.data.get("request", {}).get(key, ""))
            if value == "":
                self.query_one(f"#request-{key}-container").add_class("hidden")
                self.query_one(f"#request-{key}").add_class("hidden")
                self.query_one(f"#request-{key}", Static).update("")
            else:
                self.query_one(f"#request-{key}", Static).update(value)
                self.query_one(f"#request-{key}-container").remove_class("hidden")
                self.query_one(f"#request-{key}").remove_class("hidden")
        self.query_one(f"#response-text", Static).update(
            self.data["response"]["choices"][0]["text"]
        )

    def compose(self) -> ComposeResult:
        """Create child widgets of a Request."""
        yield Container(
            Static(id="id", classes="id"),
            Static(id="date", classes="date"),
            id="id-container",
        )
        request_containers = [
            Container(
                Static(id=f"request-{key}", classes=f"request-{key}"),
                id=f"request-{key}-container",
            )
            for key in REQUEST_KEYS
        ]
        yield Static("REQUEST", id="request-header")
        yield Container(*request_containers, id="requests")
        yield Static("RESULTS", id="results-header")
        yield Container(
            Static(id="response-text", classes="response-text"),
            id="response-text-container",
        )
        yield PromptSidebar()

    def copy_to_clipboard(self):
        pyperclip.copy(self.data["response"]["choices"][0]["text"])


class RequestApp(App):
    """A Textual app to manage requests."""

    CSS_PATH = Path("__file__").parent / "app.css"
    BINDINGS = [tuple(b.values()) for b in config["lockhart"]["tui"]["bindings"]]

    @property
    def history(self):
        try:
            return self._history
        except AttributeError:
            self.update_history()
            return self._history

    def update_history(self):
        self._history = load_history()
        self._history.reverse()
        self._history

    def on_mount(self):
        try:
            self.query_one(Request).update_data()
        except NoMatches:
            console.log("could not find a Request")
        self.i = 0

    def compose(self) -> ComposeResult:
        """Create child widgets for the app."""
        self.i = 0
        try:
            yield Container(
                Request(id=self.i, data=self.history[self.i]),
                id="timers",
            )
        except IndexError:
            self.log("history is empty")
            yield Container(
                Request(id=self.i, data=None),
                id="timers",
            )
        yield Footer()

    def action_next(self):
        self.activate(1)

    def action_prev(self):
        self.activate(-1)

    def action_copy_to_clipboard(self):
        self.query_one(Request).copy_to_clipboard()

    def activate(self, n=0, i=None):
        self.i = self.i + n
        if self.i > len(self.history) - 1:
            self.i = 0
        if self.i < 0:
            self.i = len(self.history) - 1
        try:
            self.query_one(Request).data = self.history[self.i]
            self.query_one(Request).myid = self.i
            self.query_one(Request).update_data()
        except IndexError:
            console.log("history is empty")

    def action_toggle_dark(self) -> None:
        """An action to toggle dark mode."""
        self.dark = not self.dark

    def action_edit(self):
        prompt = self.history[self.i]["request"]
        if "input" in prompt.keys():
            prompt["api"] = "Edit"
            prompt["input"] = self.history[self.i]["response"]["choices"][0]["text"]
        else:
            prompt["api"] = "Edit"
            prompt["engine"] = "code-davinci-edit-001"
            prompt["instruction"] = ""
            prompt["input"] = self.history[self.i]["response"]["choices"][0]["text"]
            remove_keys = [
                "prompt",
                "frequency_penalty",
                "presence_penalty",
                "max_tokens",
            ]
            for key in remove_keys:
                if key in prompt.keys():
                    prompt.pop(key)
        self._driver.stop_application_mode()
        run_prompt(prompt, edit=True)
        self._driver.start_application_mode()
        self.update_history()
        self.activate(i=0)

    def action_run_prompt(self):
        prompt = self.query_one(Request).data["request"]
        self._driver.stop_application_mode()
        run_prompt(prompt, edit=True)
        self._driver.start_application_mode()
        self.update_history()
        self.activate(i=0)

    def action_new_commit(self):
        self._driver.stop_application_mode()
        run_prompt("commit", edit=True)
        self._driver.start_application_mode()
        self.update_history()
        self.activate(i=0)

    def action_new_code_edit(self):
        self._driver.stop_application_mode()
        run_prompt("code-edit", edit=True)
        self._driver.start_application_mode()
        self.update_history()
        self.activate(i=0)

    def action_new_code_create(self):
        self._driver.stop_application_mode()
        run_prompt("code-create", edit=True)
        self._driver.start_application_mode()
        self.update_history()
        self.activate(i=0)

    def action_toggle_sidebar(self):
        try:
            self.query_one("PromptSidebar").remove()
        except NoMatches:
            self.mount(PromptSidebar())

    def on_button_pressed(self, event: Button.Pressed) -> None:
        prompt = event.button.id[:-7]
        self._driver.stop_application_mode()
        self.log("running prompt")
        self.log(prompt)
        result = run_prompt(prompt, edit=True)
        self.log(result)
        self._driver.start_application_mode()
        self.update_history()
        self.activate(i=0)


def run_app():
    import os
    import sys

    from textual.features import parse_features

    dev = "--dev" in sys.argv
    features = set(parse_features(os.environ.get("TEXTUAL", "")))
    if dev:
        features.add("debug")
        features.add("devtools")

    os.environ["TEXTUAL"] = ",".join(sorted(features))
    app = RequestApp()
    app.run()


if __name__ == "__main__":
    run_app()



from lockhart.config import get_config
from pathlib import Path


def test_get_config_default(mocker):
    mocker.patch(
        "pathlib.Path.home",
        return_value=Path(__file__).parents[1] / "lockhart" / "sample-lockhart.toml",
    )
    mocker.patch("pathlib.Path.read_text", return_value='[lockhart]\nname = "Lockhart"')
    config = get_config()
    assert config["lockhart"]["name"] == "Lockhart"


def test_get_config_custom():
    config_file = Path(__file__).parents[1] / "lockhart" / "sample-lockhart.toml"
    config = get_config(config_file)
    assert config["lockhart"]["name"] == "Lockhart"



from unittest.mock import patch

from lockhart.tui.app import RequestApp


async def test_run_app(load_history):
    async with RequestApp().run_test() as request_app:
        assert request_app.app._running
        assert load_history.call_count == 1


async def test_press_next(load_history):
    async with RequestApp().run_test() as request_app:
        await request_app.press("j")
        assert request_app.app.i == 1
        await request_app.press("j")
        assert request_app.app.i == 2
        await request_app.press("j")
        assert request_app.app.i == 3
        assert load_history.call_count == 1
        assert load_history.called_with()


async def test_press_prev(load_history):
    async with RequestApp().run_test() as request_app:
        await request_app.press("k")
        assert request_app.app.i == len(request_app.app.history) - 1
        await request_app.press("k")
        assert request_app.app.i == len(request_app.app.history) - 2
        await request_app.press("k")
        assert request_app.app.i == len(request_app.app.history) - 3
        assert load_history.call_count == 1
        assert load_history.called_with()


async def test_press_next_rollover(load_history):
    async with RequestApp().run_test() as request_app:
        for _ in range(len(request_app.app.history)):
            await request_app.press("j")
        assert request_app.app.i == 0
        assert load_history.call_count == 1
        assert load_history.called_with()


async def test_copy_to_clipboard(load_history):
    async with RequestApp().run_test() as request_app:
        with patch(
            "lockhart.tui.app.Request.copy_to_clipboard"
        ) as mock_copy_to_clipboard:
            await request_app.press("y")
            assert load_history.call_count == 1
            assert load_history.called_with()
            assert mock_copy_to_clipboard.call_count == 1
            assert mock_copy_to_clipboard.called_with()


async def test_toggle_dark(load_history):
    async with RequestApp().run_test() as request_app:
        assert request_app.app.dark
        await request_app.press("d")
        assert not request_app.app.dark
        assert load_history.call_count == 1
        assert load_history.called_with()



import typer
from rich.console import Console

from lockhart.cli.common import verbose_callback
from lockhart.history import load_history

history_app = typer.Typer()


@history_app.callback()
def history(
    verbose: bool = typer.Option(
        False,
        callback=verbose_callback,
        help="show the log messages",
    ),
):
    "history cli"


@history_app.command()
def list(
    verbose: bool = typer.Option(
        False,
        callback=verbose_callback,
        help="show the log messages",
    ),
):
    """
    List the history of the commands.
    """
    Console().print(load_history())



import os
from pathlib import Path
import pickle

import pytest

from lockhart.history import load_history, save_history


@pytest.fixture
def history_file(tmpdir):
    return tmpdir.join("history.pkl")


def test_save_history(history_file):
    history = ["test1", "test2"]
    save_history(history, history_file)
    assert os.path.exists(str(history_file))
    with open(str(history_file), "rb") as f:
        saved_history = pickle.load(f)
    assert saved_history == history


def test_load_history(history_file):
    history = ["test1", "test2"]
    with open(str(history_file), "wb") as f:
        pickle.dump(history, f)
    loaded_history = load_history(str(history_file))
    assert loaded_history == history


def test_load_history_empty(history_file):
    loaded_history = load_history(str(history_file))
    assert loaded_history == []


@pytest.fixture
def history_file_pathlib(tmpdir):
    return Path(tmpdir.join("history.pkl"))


def test_save_history_pathlib(history_file_pathlib):
    history = ["test1", "test2"]
    save_history(history, history_file_pathlib)
    assert os.path.exists(history_file_pathlib)
    with open(history_file_pathlib, "rb") as f:
        saved_history = pickle.load(f)
    assert saved_history == history


def test_load_history_pathlib(history_file_pathlib):
    history = ["test1", "test2"]
    with open(history_file_pathlib, "wb") as f:
        pickle.dump(history, f)
    loaded_history = load_history(history_file_pathlib)
    assert loaded_history == history


def test_load_history_empty_pathlib(history_file_pathlib):
    loaded_history = load_history(history_file_pathlib)
    assert loaded_history == []



import typer
from rich.console import Console

from lockhart.cli.common import verbose_callback
from lockhart.config import config as configuration

config_app = typer.Typer()


@config_app.callback()
def config(
    verbose: bool = typer.Option(
        False,
        callback=verbose_callback,
        help="show the log messages",
    ),
):
    "configuration cli"


@config_app.command()
def show(
    verbose: bool = typer.Option(
        False,
        callback=verbose_callback,
        help="show the log messages",
    ),
):
    Console().print(configuration)



# lockhart/config.py
from pathlib import Path

import tomlkit

from lockhart.console import console


def get_config(
    config_file=Path.home() / ".config" / "lockhart" / "lockhart.toml",
    default_config_file=Path(__file__).parent / "sample-lockhart.toml",
):
    try:
        config = tomlkit.loads(config_file.read_text())
    except FileNotFoundError:
        console.log(f"No config file found at {config_file}. Using default config.")
        config_file = default_config_file
        config = tomlkit.loads(config_file.read_text())

    return config


config = get_config()



import typer
from rich.console import Console

from lockhart import prompts
from lockhart.cli.common import verbose_callback
from lockhart.config import config as configuration
from lockhart.console import console

prompt_app = typer.Typer()


@prompt_app.callback(invoke_without_command=True)
def prompt(
    verbose: bool = typer.Option(
        False,
        callback=verbose_callback,
        help="show the log messages",
    ),
):
    "prompt cli"


@prompt_app.command()
def list(
    verbose: bool = typer.Option(
        False,
        callback=verbose_callback,
        help="show the log messages",
    ),
) -> None:
    for prompt in configuration.get("prompts", {}).keys():
        Console().print(prompt)


@prompt_app.command()
def run(
    prompt: str = typer.Argument(..., help="the configured prompt to run"),
    dry_run: bool = typer.Option(False, help="run without sending the prompt"),
    edit: bool = typer.Option(
        False, help="open the resulting prompt in your editor before running"
    ),
    verbose: bool = typer.Option(
        False,
        callback=verbose_callback,
        help="show the log messages",
    ),
):
    console.log(f"running configured prompt: {prompt}")
    result = prompts.run_prompt(prompt, dry_run, edit)
    if result:
        try:
            Console().print(result["choices"][0]["text"])
        except KeyError:
            Console().print(result)
        except KeyError:
            Console().print(result)
        except KeyError:
            Console().print(result)



import copy
import os
import shutil
import subprocess
import sys
import tempfile
from datetime import datetime
from pathlib import Path
from typing import Optional, Union

import openai
import tomlkit

from lockhart.config import config
from lockhart.console import console
from lockhart.history import load_history, save_history
from lockhart.jinja_env import get_jinja_env


def load_prompt(prompt: str) -> dict:
    """
    Loads the configuration for the prompt from config.  Finds the
    corresponding profile and unpacks the prompt config into that profile.
    """
    profile_name = config["prompts"][prompt].get("profile")
    prompt_config = config["prompts"][prompt]
    if profile_name is None or profile_name not in config["profiles"]:
        return prompt_config
    profile = copy.deepcopy(config["profiles"][profile_name])
    for key, value in prompt_config.items():
        if key != "profile":
            profile.update({key: value})
    return profile


def run_prompt(
    prompt_name: Union[str, dict], dry_run: bool = False, edit: bool = False
) -> Optional[str]:
    console.log("running prompt")
    text = ""

    console.log("getting stdin")

    if not sys.stdin.isatty():
        # if nothing is piped in, sys.stin.read will block forever
        text = sys.stdin.read()

    console.log(f"read stdin: {text}")

    if isinstance(prompt_name, str):
        prompt = load_prompt(prompt_name)
    else:
        prompt = prompt_name

    prompt = template_prompt(prompt, text)

    if edit:
        return edit_prompt(prompt)

    if prompt is None:
        return "prompt quit"

    if dry_run:
        console.log("dry run enabled, returning prompt")
        return prompt

    console.log("prompt: ", prompt)
    console.log("running completion")
    # response = openai.Completion.create(**prompt)
    if prompt.get("api", None):
        api = getattr(openai, prompt.pop("api"))
    elif "prompt" in prompt.keys():
        api = getattr(openai, "Completion")
    else:
        api = getattr(openai, "Edit")

    response = api.create(**prompt)
    text = response["choices"][0]["text"]
    history = load_history()
    history.append(
        {
            "prompt_name": prompt_name,
            "datetime": datetime.now(),
            "request": prompt,
            "response": response,
        }
    )
    save_history(history)
    return response


def template_prompt(prompt: dict, text: str) -> dict:
    for key in prompt:
        console.log(f"templating {key}: {prompt[key].__repr__()}")

        if isinstance(prompt[key], str):
            template = get_jinja_env().from_string(prompt[key])
            value = template.render(input=input, text=text)
            prompt.update(
                {
                    key: tomlkit.string(
                        f"\n{value}\n" if "\n" in value else value,
                        multiline=("\n" in value),
                    )
                }
            )
    return prompt


def edit_prompt(prompt: dict) -> dict:
    editor = os.environ.get("EDITOR", "vim")
    if not shutil.which(editor):
        if os.name == "nt":
            raise FileNotFoundError(
                f"your configured %EDITOR% `{editor}` was not found on your %PATH%"
            )
        else:
            raise FileNotFoundError(
                f"your configured $EDITOR `{editor}` was not found on your $PATH"
            )

    console.log(f"editing prompt with {editor}")
    with tempfile.NamedTemporaryFile(prefix="lockhart", suffix=".toml") as file:
        file.seek(0)
        file.write(("# edit = true\n" + tomlkit.dumps(prompt)).encode())
        file.flush()
        st_mtime = os.stat(file.name).st_mtime  # create time by lockhart
        try:
            proc = subprocess.Popen([editor, file.name])
        except FileNotFoundError:
            console.log(f"{editor} is not installed")
            return
        proc.wait()
        if os.stat(file.name).st_mtime == st_mtime:
            # the file was not modified, return without running
            return

        prompt = tomlkit.loads(Path(file.name).read_text())
    console.log(f"updated prompt\n{prompt}")
    dry_run = bool(prompt.pop("dry_run", False))
    edit = bool(prompt.pop("edit", False))
    return run_prompt(prompt, dry_run=dry_run, edit=edit)


def list_args(func: callable):
    """lists the arguments that the function takes"""
    args = func.__code__.co_varnames
    return args
    return list(func.__code__.co_varnames)



import typer

from lockhart.cli.common import verbose_callback
from lockhart.tui.app import run_app

tui_app = typer.Typer()


@tui_app.callback(invoke_without_command=True)
def tui(
    verbose: bool = typer.Option(
        False,
        callback=verbose_callback,
        help="show the log messages",
    ),
):
    run_app()
